'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var repositoryProvider = require('repository-provider');
var execa = require('execa');
var path = require('path');
var makeDir = require('make-dir');
var globby = require('globby');
var fs = require('fs');
var os = require('os');

const {
  stat
} = require("fs").promises;
class LocalRepository extends repositoryProvider.Repository {
  static get defaultOptions() {
    return Object.assign({
      workspace: "/tmp"
    }, super.defaultOptions);
  }
  get execOptions() {
    return {
      cwd: this.workspace
    };
  }
  async _initialize(workspace) {
    await super._initialize();
    try {
      await stat(this.workspace);
      const remoteResult = await execa("git", ["remote", "-v"], this.execOptions);
      console.log(remoteResult.stdout);
      const m = remoteResult.stdout.match(/origin\s+([^\s]+)\s+/);
      if (m && m[1] === this.name) {
        const result = await execa("git", ["pull"], this.execOptions);
      } else {
        throw new Error(`Unknown content in ${this.workspace}`);
      }
    } catch (e) {
      if (e.code === "ENOENT") {
        const result = await execa("git", ["clone", ...this.provider.cloneOptions, this.name, this.workspace]);
      } else {
        throw e;
      }
    }
    await this.initializeBranches();
  }
  async initializeBranches() {
    const result = await execa("git", ["branch", "--list"], this.execOptions);
    result.stdout.split(/\n/).forEach(b => {
      const m = b.match(/^(\*\s+)?([^\s]+)/);
      if (m) {
        const name = m[2];
        const branch = new this.provider.branchClass(this, name);
        this._branches.set(branch.name, branch);
      }
    });
  }
  get urls() {
    return [this.name];
  }
  async push() {
    return execa("git", ["push"], this.execOptions);
  }
  async createBranch(name, from) {
    const result = await execa("git", ["checkout", "-b", name], this.execOptions);
    const b = new this.provider.branchClass(this, name);
    this._branches.set(b.name, b);
    return b;
  }
  async deleteBranch(name) {
    await execa("git", ["checkout", "master"], this.execOptions);
    await execa("git", ["branch", "-D", name], this.execOptions);
    this._branches.delete(name);
  }
  async refId(ref) {
    const g = await execa("git", ["show-ref", ref], this.execOptions);
    return g.stdout.split(/\s+/)[0];
  }
}

function _AwaitValue(value) {
  this.wrapped = value;
}

function _AsyncGenerator(gen) {
  var front, back;

  function send(key, arg) {
    return new Promise(function (resolve, reject) {
      var request = {
        key: key,
        arg: arg,
        resolve: resolve,
        reject: reject,
        next: null
      };

      if (back) {
        back = back.next = request;
      } else {
        front = back = request;
        resume(key, arg);
      }
    });
  }

  function resume(key, arg) {
    try {
      var result = gen[key](arg);
      var value = result.value;
      var wrappedAwait = value instanceof _AwaitValue;
      Promise.resolve(wrappedAwait ? value.wrapped : value).then(function (arg) {
        if (wrappedAwait) {
          resume("next", arg);
          return;
        }

        settle(result.done ? "return" : "normal", arg);
      }, function (err) {
        resume("throw", err);
      });
    } catch (err) {
      settle("throw", err);
    }
  }

  function settle(type, value) {
    switch (type) {
      case "return":
        front.resolve({
          value: value,
          done: true
        });
        break;

      case "throw":
        front.reject(value);
        break;

      default:
        front.resolve({
          value: value,
          done: false
        });
        break;
    }

    front = front.next;

    if (front) {
      resume(front.key, front.arg);
    } else {
      back = null;
    }
  }

  this._invoke = send;

  if (typeof gen.return !== "function") {
    this.return = undefined;
  }
}

if (typeof Symbol === "function" && Symbol.asyncIterator) {
  _AsyncGenerator.prototype[Symbol.asyncIterator] = function () {
    return this;
  };
}

_AsyncGenerator.prototype.next = function (arg) {
  return this._invoke("next", arg);
};

_AsyncGenerator.prototype.throw = function (arg) {
  return this._invoke("throw", arg);
};

_AsyncGenerator.prototype.return = function (arg) {
  return this._invoke("return", arg);
};

function _wrapAsyncGenerator(fn) {
  return function () {
    return new _AsyncGenerator(fn.apply(this, arguments));
  };
}

function _awaitAsyncGenerator(value) {
  return new _AwaitValue(value);
}

const {
  readFile,
  writeFile
} = require("fs").promises;
class LocalBranch extends repositoryProvider.Branch {
  get workspace() {
    return this.repository.workspace;
  }
  get execOptions() {
    return this.repository.execOptions;
  }
  async content(fileName, options = {
    encoding: "utf8"
  }) {
    return new repositoryProvider.Content(fileName, (await readFile(path.join(this.workspace, fileName), options)));
  }
  async write(updates) {
    await Promise.all(updates.map(b => makeDir(path.dirname(path.join(this.workspace, b.path)))));
    return new Promise((resolve, reject) => {
      let ongoing = 0;
      for (const u of updates) {
        const o = fs.createWriteStream(path.join(this.workspace, u.path));
        o.on('error', error => reject(error));
        o.on("finish", () => {
          ongoing--;
          if (ongoing <= 0) {
            resolve();
          }
        });
        u.toStream().pipe(o);
        ongoing++;
      }
    });
  }
  async commit(message, updates, options) {
    await this.write(updates);
    await execa("git", ["add", ...updates.map(b => b.path)], this.execOptions);
    await execa("git", ["commit", "-m", message], this.execOptions);
    await execa("git", ["push", "--set-upstream", "origin", this.name], this.execOptions);
  }
  list(matchingPatterns = ["**/.*", "**/*"]) {
    var _this = this;
    return _wrapAsyncGenerator(function* () {
      for (const entry of yield _awaitAsyncGenerator(globby(matchingPatterns, {
        cwd: _this.workspace
      }))) {
        yield new repositoryProvider.Content(entry);
      }
    })();
  }
  async createPullRequest(to, message) {
    return new this.provider.pullRequestClass(this.repository, "0", {
      title: "please create pull request manually"
    });
  }
}

const {
  stat: stat$1
} = require("fs").promises;
class LocalProvider extends repositoryProvider.Provider {
  static optionsFromEnvironment(env) {
    if (env.GIT_CLONE_OPTIONS !== undefined) {
      return {
        cloneOptions: env.GIT_CLONE_OPTIONS.split(/\s+/)
      };
    }
    return undefined;
  }
  static get defaultOptions() {
    return {
      cloneOptions: [],
      workspace: os.tmpdir()
    };
  }
  get workspace() {
    return this.config.workspace;
  }
  get repositoryClass() {
    return LocalRepository;
  }
  get branchClass() {
    return LocalBranch;
  }
  async newWorkspacePath() {
    do {
      this._nextWorkspace = this._nextWorkspace === undefined ? 1 : this._nextWorkspace + 1;
      let w = path.join(this.workspace, `r${this._nextWorkspace}`);
      try {
        const s = await stat$1(w);
      } catch (e) {
        return w;
      }
    } while (true);
  }
  async repository(name) {
    if (name === undefined) {
      return undefined;
    }
    let repository = this.repositories.get(name);
    if (repository === undefined) {
      repository = new this.repositoryClass(this, name, {
        workspace: await this.newWorkspacePath()
      });
      this.repositories.set(repository.name, repository);
    }
    return repository;
  }
}

exports.LocalProvider = LocalProvider;
